#!/usr/bin/env python
## Nicolas Servant
## April 2018

""" MultiQC module to parse output from HiC-Pro """

from __future__ import print_function
from collections import OrderedDict
import os.path
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """HiC-Pro module, parses log and stats files saved by HiC-Pro."""

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name="HiC-Pro",
            anchor="hicpro",
            href="https://github.com/nservant/HiC-Pro",
            info="is an efficient and flexible pipeline for Hi-C data processing. The MultiQC module is supported since HiC-Pro v2.11.0.",
            doi="10.1186/s13059-015-0831-x",
        )

        # Find and load any HiC-Pro summary reports
        self.hicpro_data = dict()
        for k in ["mmapstat", "mpairstat", "mergestat", "mRSstat", "assplit"]:
            for f in self.find_log_files("hicpro/{}".format(k)):
                self.parse_hicpro_stats(f, k)

        # Update current statistics
        for s_name in self.hicpro_data:
            data = self.hicpro_data[s_name]
            try:
                data["duplicates"] = data["valid_interaction"] - data["valid_interaction_rmdup"]
                data["percent_duplicates"] = float(data["duplicates"]) / float(data["valid_interaction"]) * 100.0
                data["percent_mapped_R1"] = float(data["mapped_R1"]) / float(data["total_R1"]) * 100.0
                data["percent_mapped_R2"] = float(data["mapped_R2"]) / float(data["total_R2"]) * 100.0
                data["paired_reads"] = int(data["Reported_pairs"])
                data["percent_paired_reads"] = float(data["Reported_pairs"]) / float(data["total_R1"]) * 100.0
                data["percent_valid"] = float(data["valid_interaction"]) / float(data["total_R1"]) * 100.0
                data["Failed_To_Align_Read_R1"] = int(data["total_R1"]) - int(data["mapped_R1"])
                data["Failed_To_Align_Read_R2"] = int(data["total_R2"]) - int(data["mapped_R2"])
            except KeyError as e:
                log.error(f"Missing expected key {e} in sample '{s_name}'")

            try:
                data["valid_pairs_off_target"] = int(data["valid_interaction"]) - int(data["valid_pairs_on_target"])
            except KeyError:
                pass

        # Filter to strip out ignored sample names
        self.hicpro_data = self.ignore_samples(self.hicpro_data)

        if len(self.hicpro_data) == 0:
            raise UserWarning

        log.info("Found {} HiC-Pro reports".format(len(self.hicpro_data)))

        # Write parsed data to a file
        self.write_data_file(self.hicpro_data, "multiqc_hicpro")

        # Basic Stats Table
        self.hicpro_stats_table()

        # Report sections
        self.add_section(
            name="Read Mapping",
            anchor="hicpro-mapping",
            description="Alignment of reads in single-end mode.",
            helptext="""
            HiC-Pro uses a two steps mapping strategy. End-to-end mapping is first
            performed (*Full reads Alignments*). Unmapped reads are then trimmed
            at their ligation site, and the 5\' end is re-aligned on the reference
            genome (*Trimmed Reads Alignments*)""",
            plot=self.hicpro_mapping_chart(),
        )

        self.add_section(
            name="Read Pairing",
            anchor="hicpro-pairing",
            description="Pairing of single-end mapping.",
            helptext="""
            HiC-Pro combines aligned reads as pairs. Singleton, low quality pairs or
            pairs involving multi-mapped reads are usually discarded. Note that the filtering
            at pairing level can change accrding to the parameters used.""",
            plot=self.hicpro_pairing_chart(),
        )

        self.add_section(
            name="Read Pair Filtering",
            anchor="hicpro-filtering",
            description="Filtering of read pairs.",
            helptext="""
            Aligned read pairs are filtered to select valid 3C products from two
            different restriction fragments. Read pairs coming from the same fragments,
            such as self-ligation or unligated (danging-end) fragments, are discarded.
            Ligation products involving neighboring restriction fragment (religation) are also discarded.
            Filtered pairs correspond to ligation products discarded based on the range of insert/fragment sizes defined
            in the analysis. Finally, as the ligation should be a random process, valid read pairs from all
            orientations (R=Reverse, F=forward) are expected to be observed in the same proportion.""",
            plot=self.hicpro_filtering_chart(),
        )

        self.add_section(
            name="Contact Statistics",
            anchor="hicpro-rmdup",
            description="Contacts statistics after duplicates removal.",
            helptext="""
            Description of contact frequency after duplicates removal. Intra-chromosomal
            (*cis*) interaction are expected to be more frequent than inter-chromosomal contacts (*trans*)""",
            plot=self.hicpro_contact_chart(),
        )

        allele_plot = self.hicpro_as_chart()
        if allele_plot:
            self.add_section(
                name="Allele-specific analysis",
                anchor="hicpro-asan",
                description="Assignment of valid interactions (afer duplicates removal) to parental alleles.",
                helptext="""
                Description of allele-specific contacts. Valid interactions (0-1 and 1-1, resp. 0-2, 2-2)
                are used to build the genome1 (resp. genome2) parental maps.""",
                plot=allele_plot,
            )

        capture_plot = self.hicpro_capture_chart()
        if capture_plot:
            self.add_section(
                name="Capture analysis",
                anchor="hicpro-cap",
                description="Selection of interactions overlaping the targeted region(s).",
                helptext="""
                Description of capture efficiency. Valid interactions with either two (capture-capture) or
                one (capture-reporter) interactors overlapping with the target(s) are reported.""",
                plot=capture_plot,
            )

    def parse_hicpro_stats(self, f, rsection):
        """Parse a HiC-Pro stat file"""
        s_name = self.clean_s_name(os.path.basename(f["root"]), root=os.path.dirname(f["root"]))
        if s_name not in self.hicpro_data.keys():
            self.hicpro_data[s_name] = {}

        self.add_data_source(f, s_name, section=rsection)
        for l in f["f"].splitlines():
            if not l.startswith("#"):
                s = l.split("\t")
                if s[0] in self.hicpro_data[s_name]:
                    log.debug("Duplicated keys found! Overwriting: {}".format(s[0]))
                self.hicpro_data[s_name][s[0]] = int(s[1])

    def hicpro_stats_table(self):
        """Add HiC-Pro stats to the general stats table"""
        headers = OrderedDict()

        headers["percent_duplicates"] = {
            "title": "% Duplicates",
            "description": "Percent of duplicated valid pairs (%)",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "scale": "YlOrRd",
            "hidden": True,
        }

        headers["valid_interaction_rmdup"] = {
            "title": "{} Valid Pairs Unique".format(config.read_count_prefix),
            "description": "Number of valid pairs after duplicates removal ({})".format(config.read_count_desc),
            "min": 0,
            "scale": "RdYlBu",
            "modify": lambda x: x * config.read_count_multiplier,
            "shared_key": "read_count",
        }

        headers["percent_valid"] = {
            "title": "% Valid Pairs",
            "description": "Percentage of valid pairs over reported ones (%)",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "scale": "YlGn",
            "hidden": True,
        }

        headers["valid_interaction"] = {
            "title": "{} Valid Pairs".format(config.read_count_prefix),
            "description": "Number of valid pairs ({})".format(config.read_count_desc),
            "min": 0,
            "scale": "RdYlBu",
            "modify": lambda x: x * config.read_count_multiplier,
            "shared_key": "read_count",
            "hidden": True,
        }

        headers["percent_paired_reads"] = {
            "title": "% Reported",
            "description": "Percentage of paired reads (%) passing the mapping filters",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "scale": "YlGn",
            "hidden": True,
        }

        headers["paired_reads"] = {
            "title": "Reported Read Pairs",
            "description": "Total number of read pairs ({}) passing the mapping filters".format(config.read_count_desc),
            "min": "0",
            "scale": "RdYlBu",
            "modify": lambda x: x * config.read_count_multiplier,
            "shared_key": "read_count",
        }

        headers["percent_mapped_R2"] = {
            "title": "% Aligned [R2]",
            "description": "Percentage of aligned reads [R2] (%)",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "scale": "YlGn",
            "hidden": True,
        }

        headers["mapped_R2"] = {
            "title": "Aligned [R2]",
            "description": "Total number of aligned reads [R2] ({})".format(config.read_count_desc),
            "min": "0",
            "scale": "RdYlBu",
            "modify": lambda x: x * config.read_count_multiplier,
            "shared_key": "read_count",
            "hidden": True,
        }

        headers["percent_mapped_R1"] = {
            "title": "% Aligned [R1]",
            "description": "Percentage of aligned reads [R1] (%)",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "scale": "YlGn",
            "hidden": True,
        }

        headers["mapped_R1"] = {
            "title": "Aligned [R1]",
            "description": "Total number of aligned reads [R1] ({})".format(config.read_count_desc),
            "min": "0",
            "scale": "RdYlBu",
            "modify": lambda x: x * config.read_count_multiplier,
            "shared_key": "read_count",
            "hidden": True,
        }

        headers["total_R1"] = {
            "title": "Total",
            "description": "Total Number of Read Pairs",
            "min": "0",
            "scale": "RdYlBu",
            "modify": lambda x: x * config.read_count_multiplier,
            "shared_key": "read_count",
        }

        self.general_stats_addcols(self.hicpro_data, headers, "HiC-Pro")

    def hicpro_mapping_chart(self):
        """Generate the HiC-Pro Aligned reads plot"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys["Full_Alignments_Read"] = {"color": "#005ce6", "name": "Full reads Alignments"}
        keys["Trimmed_Alignments_Read"] = {"color": "#3385ff", "name": "Trimmed reads Alignments"}
        keys["Failed_To_Align_Read"] = {"color": "#a9a2a2", "name": "Failed To Align"}

        data = [{}, {}]
        for s_name in self.hicpro_data:
            for r in [1, 2]:
                try:
                    data[r - 1]["{} [R{}]".format(s_name, r)] = {
                        "Full_Alignments_Read": self.hicpro_data[s_name]["global_R{}".format(r)],
                        "Trimmed_Alignments_Read": self.hicpro_data[s_name]["local_R{}".format(r)],
                        "Failed_To_Align_Read": int(self.hicpro_data[s_name]["total_R{}".format(r)])
                        - int(self.hicpro_data[s_name]["mapped_R{}".format(r)]),
                    }
                except KeyError as e:
                    log.error(f"Missing expected plot key {e} in {s_name} Read {r}")

        # Config for the plot
        config = {
            "id": "hicpro_mapping_stats_plot",
            "title": "HiC-Pro: Mapping Statistics",
            "ylab": "# Reads",
            "ylab": "# Reads: Read 1",
            "data_labels": [
                {"name": "Read 1", "ylab": "# Reads: Read 1"},
                {"name": "Read 2", "ylab": "# Reads: Read 2"},
            ],
        }

        return bargraph.plot(data, [keys, keys], config)

    def hicpro_pairing_chart(self):
        """Generate Pairing chart"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys["Unique_paired_alignments"] = {"color": "#005ce6", "name": "Uniquely Aligned"}
        keys["Low_qual_pairs"] = {"color": "#b97b35", "name": "Low Quality"}
        keys["Pairs_with_singleton"] = {"color": "#ff9933", "name": "Singleton"}
        keys["Multiple_pairs_alignments"] = {"color": "#e67300", "name": "Multi Aligned"}
        keys["Unmapped_airs"] = {"color": "#a9a2a2", "name": "Failed To Align"}

        # Config for the plot
        config = {
            "id": "hicpro_pairing_stats_plot",
            "title": "HiC-Pro: Pairing Statistics",
            "ylab": "# Reads",
            "cpswitch_counts_label": "Number of Reads",
        }

        return bargraph.plot(self.hicpro_data, keys, config)

    def hicpro_filtering_chart(self):
        """Generate the HiC-Pro filtering plot"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys["Valid_interaction_pairs_FF"] = {"color": "#ccddff", "name": "Valid Pairs FF"}
        keys["Valid_interaction_pairs_RR"] = {"color": "#6699ff", "name": "Valid Pairs RR"}
        keys["Valid_interaction_pairs_RF"] = {"color": "#0055ff", "name": "Valid Pairs RF"}
        keys["Valid_interaction_pairs_FR"] = {"color": "#003399", "name": "Valid Pairs FR"}
        keys["Self_Cycle_pairs"] = {"color": "#ffad99", "name": "Same Fragment - Self-Circle"}
        keys["Dangling_end_pairs"] = {"color": "#ff5c33", "name": "Same Fragment - Dangling Ends"}
        keys["Religation_pairs"] = {"color": "#cc2900", "name": "Re-ligation"}
        keys["Filtered_pairs"] = {"color": "#661400", "name": "Filtered pairs"}
        keys["Dumped_pairs"] = {"color": "#330a00", "name": "Dumped pairs"}

        # Config for the plot
        config = {
            "id": "hicpro_filtering_plot",
            "title": "HiC-Pro: Filtering Statistics",
            "ylab": "# Read Pairs",
            "cpswitch_counts_label": "Number of Read Pairs",
        }

        return bargraph.plot(self.hicpro_data, keys, config)

    def hicpro_contact_chart(self):
        """Generate the HiC-Pro interaction plot"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys["cis_shortRange"] = {"color": "#0039e6", "name": "Unique: cis <= 20Kbp"}
        keys["cis_longRange"] = {"color": "#809fff", "name": "Unique: cis > 20Kbp"}
        keys["trans_interaction"] = {"color": "#009933", "name": "Unique: trans"}
        keys["duplicates"] = {"color": "#a9a2a2", "name": "Duplicate read pairs"}

        # Config for the plot
        config = {
            "id": "hicpro_contact_plot",
            "title": "HiC-Pro: Contact Statistics",
            "ylab": "# Pairs",
            "cpswitch_counts_label": "Number of Pairs",
        }

        return bargraph.plot(self.hicpro_data, keys, config)

    def hicpro_as_chart(self):
        """Generate Allele-specific plot"""

        keys = OrderedDict()
        keys["Valid_pairs_from_ref_genome_(1-1)"] = {"color": "#e6550d", "name": "Genome1 specific read pairs (1-1)"}
        keys["Valid_pairs_from_ref_genome_with_one_unassigned_mate_(0-1/1-0)"] = {
            "color": "#fdae6b",
            "name": "Genome1 with one unassigned mate (0-1/1-0)",
        }
        keys["Valid_pairs_from_alt_genome_(2-2)"] = {"color": "#756bb1", "name": "Genome2 specific read pairs (2-2)"}
        keys["Valid_pairs_from_alt_genome_with_one_unassigned_mate_(0-2/2-0)"] = {
            "color": "#bcbddc",
            "name": "Genome2 with one unassigned mate (0-2/2-0)",
        }
        keys["Valid_pairs_from_alt_and_ref_genome_(1-2/2-1)"] = {
            "color": "#a6611a",
            "name": "Trans homologuous read pairs (1-2/2/1)",
        }
        keys["Valid_pairs_with_both_unassigned_mated_(0-0)"] = {"color": "#cccccc", "name": "Unassigned read pairs"}
        keys["Valid_pairs_with_at_least_one_conflicting_mate_(3-)"] = {
            "color": "#a9a2a2",
            "name": "Conflicting read pairs",
        }

        # check allele-specific analysis was run
        num_samples = 0
        for s_name in self.hicpro_data:
            for k in keys:
                num_samples += sum([1 if k in self.hicpro_data[s_name] else 0])
        if num_samples == 0:
            return False

        # Config for the plot
        config = {
            "id": "hicpro_asan_plot",
            "title": "HiC-Pro: Allele-specific Statistics",
            "ylab": "# Pairs",
            "cpswitch_counts_label": "Number of Pairs",
        }

        return bargraph.plot(self.hicpro_data, keys, config)

    def hicpro_capture_chart(self):
        """Generate Capture Hi-C plot"""

        keys = OrderedDict()
        keys["valid_pairs_on_target_cap_cap"] = {"color": "#0039e6", "name": "Capture-Capture interactions"}
        keys["valid_pairs_on_target_cap_rep"] = {"color": "#809fff", "name": "Capture-Reporter interactions"}
        keys["valid_pairs_off_target"] = {"color": "#cccccc", "name": "Off-target valid pairs"}

        # Check capture info are available
        num_samples = 0
        for s_name in self.hicpro_data:
            for k in keys:
                num_samples += sum([1 if k in self.hicpro_data[s_name] else 0])
        if num_samples == 0:
            return False

        # Config for the plot
        config = {
            "id": "hicpro_cap_plot",
            "title": "HiC-Pro: Capture Statistics",
            "ylab": "# Pairs",
            "cpswitch_counts_label": "Number of Pairs",
        }

        return bargraph.plot(self.hicpro_data, keys, config)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard WgsMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc import config
from multiqc.plots import linegraph, bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard WgsMetrics reports and parse their data"""

    # Set up vars
    self.picard_wgsmetrics_data = dict()
    self.picard_wgsmetrics_histogram = dict()
    self.picard_wgsmetrics_samplestats = dict()
    picard_config = getattr(config, "picard_config", {})
    skip_histo = picard_config.get("wgsmetrics_skip_histogram", False)

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/wgs_metrics", filehandles=True):
        s_name = None
        in_hist = False
        for l in f["f"]:

            # Catch the histogram values
            if s_name is not None and in_hist is True and not skip_histo:
                try:
                    sections = l.split("\t")
                    cov = int(sections[0])
                    count = int(sections[1])
                    self.picard_wgsmetrics_histogram[s_name][cov] = count
                except ValueError:
                    # Reset in case we have more in this log file
                    s_name = None
                    in_hist = False

            # New log starting
            if "WgsMetrics" in l and "INPUT" in l:
                s_name = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)

            if s_name is not None:
                metric_names = ["CollectWgsMetrics$WgsMetrics", "picard.analysis.WgsMetrics"]
                if any(n in l for n in metric_names) and "## METRICS CLASS" in l:
                    if s_name in self.picard_wgsmetrics_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
                    self.add_data_source(f, s_name, section="WgsMetrics")
                    self.picard_wgsmetrics_data[s_name] = dict()
                    keys = f["f"].readline().strip("\n").split("\t")
                    vals = f["f"].readline().strip("\n").split("\t")
                    if len(vals) == len(keys):
                        for i, k in enumerate(keys):
                            try:
                                self.picard_wgsmetrics_data[s_name][k] = float(vals[i])
                            except ValueError:
                                self.picard_wgsmetrics_data[s_name][k] = vals[i]

                    # Skip lines on to histogram
                    next(f["f"])
                    next(f["f"])
                    next(f["f"])

                    self.picard_wgsmetrics_histogram[s_name] = OrderedDict()
                    in_hist = True

        for key in list(self.picard_wgsmetrics_data.keys()):
            if len(self.picard_wgsmetrics_data[key]) == 0:
                self.picard_wgsmetrics_data.pop(key, None)
        for s_name in list(self.picard_wgsmetrics_histogram.keys()):
            if len(self.picard_wgsmetrics_histogram[s_name]) == 0:
                self.picard_wgsmetrics_histogram.pop(s_name, None)
                log.debug("Ignoring '{}' histogram as no data parsed".format(s_name))

    # Filter to strip out ignored sample names
    self.picard_wgsmetrics_data = self.ignore_samples(self.picard_wgsmetrics_data)

    if len(self.picard_wgsmetrics_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_wgsmetrics_data, "multiqc_picard_wgsmetrics")

        # Add to general stats table
        self.general_stats_headers["MEDIAN_COVERAGE"] = {
            "title": "Median Coverage",
            "description": "The median coverage in bases of the genome territory, after all filters are applied.",
            "min": 0,
            "suffix": "X",
            "scale": "GnBu",
        }
        self.general_stats_headers["MEAN_COVERAGE"] = {
            "title": "Mean Coverage",
            "description": "The mean coverage in bases of the genome territory, after all filters are applied.",
            "min": 0,
            "suffix": "X",
            "scale": "GnBu",
            "hidden": True,
        }
        self.general_stats_headers["SD_COVERAGE"] = {
            "title": "Median Coverage",
            "description": "The standard deviation coverage in bases of the genome territory, after all filters are applied.",
            "min": 0,
            "suffix": "X",
            "scale": "GnBu",
            "hidden": True,
        }
        # user configurable coverage level
        try:
            covs = config.picard_config["general_stats_target_coverage"]
            assert type(covs) == list
            assert len(covs) > 0
            covs = [str(i) for i in covs]
            log.debug("Custom Picard coverage thresholds: {}".format(", ".join([i for i in covs])))
        except (AttributeError, TypeError, AssertionError, KeyError):
            covs = ["30"]
        for c in covs:
            self.general_stats_headers["PCT_{}X".format(c)] = {
                "id": "picard_target_bases_{}X".format(c),
                "title": "Bases &ge; {}X".format(c),
                "description": "Percent of target bases with coverage &ge; {}X".format(c),
                "max": 100,
                "min": 0,
                "suffix": "%",
                "format": "{:,.0f}",
                "scale": "RdYlGn",
                "modify": lambda x: self.multiply_hundred(x),
            }

        for s_name in self.picard_wgsmetrics_data:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(self.picard_wgsmetrics_data[s_name])

        # Section with histogram plot
        if len(self.picard_wgsmetrics_histogram) > 0 and not skip_histo:

            # Figure out where to cut histogram tail
            max_cov = picard_config.get("wgsmetrics_histogram_max_cov")
            if max_cov is None:
                max_cov = 10
                for s_name, samp in self.picard_wgsmetrics_histogram.items():
                    total = float(sum(samp.values()))
                    running_total = 0
                    for k, v in samp.items():
                        running_total += v
                        if running_total > total * 0.99:
                            max_cov = max(k, max_cov)
                            break

            # Cut histogram tail and make a normalised percentage version of the data plus dropoff
            data = {}
            data_percent = {}
            maxval = 0
            for s_name, samp in self.picard_wgsmetrics_histogram.items():
                data[s_name] = OrderedDict()
                data_percent[s_name] = OrderedDict()
                total = float(sum(samp.values()))
                cumulative = 0
                for k, v in samp.items():
                    if k <= max_cov:
                        cumulative += v
                        data[s_name][k] = v
                        maxval = max(maxval, v)
                        data_percent[s_name][k] = 100 - (cumulative / total) * 100
                    else:
                        break

            # Plot the histogram data and add section
            pconfig = {
                "id": "picard_wgs_metrics_histogram",
                "title": "Picard: WGS Coverage",
                "ylab": "Percentage of Bases",
                "xlab": "Fold Coverage",
                "xDecimals": False,
                "tt_label": "<b>{point.x} X</b>: {point.y:.1f}",
                "ymin": 0,
                "ymax": 100,
                "smooth_points": picard_config.get("wgsmetrics_histogram_smooth", 1000),
                "data_labels": [
                    {"name": "Percentage Drop-Off", "ylab": "Percentage of Bases", "ymax": 100},
                    {"name": "Counts Histogram", "ylab": "Coverage", "ymax": maxval},
                ],
            }
            self.add_section(
                name="WGS Coverage",
                anchor="picard-wgsmetrics-cov",
                description="The number of bases in the genome territory for each fold coverage. "
                + "Note that final 1% of data is hidden to prevent very long tails.",
                plot=linegraph.plot([data_percent, data], pconfig),
            )

        # Bar plot of ignored bases
        pdata = dict()
        for s_name, data in self.picard_wgsmetrics_data.items():
            pdata[s_name] = dict()
            pdata[s_name]["PCT_EXC_MAPQ"] = data["PCT_EXC_MAPQ"] * 100.0
            pdata[s_name]["PCT_EXC_DUPE"] = data["PCT_EXC_DUPE"] * 100.0
            pdata[s_name]["PCT_EXC_UNPAIRED"] = data["PCT_EXC_UNPAIRED"] * 100.0
            pdata[s_name]["PCT_EXC_BASEQ"] = data["PCT_EXC_BASEQ"] * 100.0
            pdata[s_name]["PCT_EXC_OVERLAP"] = data["PCT_EXC_OVERLAP"] * 100.0
            pdata[s_name]["PCT_EXC_CAPPED"] = data["PCT_EXC_CAPPED"] * 100.0

        keys = OrderedDict()
        keys["PCT_EXC_MAPQ"] = {"name": "Low mapping quality"}
        keys["PCT_EXC_DUPE"] = {"name": "Duplicates reads"}
        keys["PCT_EXC_UNPAIRED"] = {"name": "No mapped mate pair"}
        keys["PCT_EXC_BASEQ"] = {"name": "Low base quality"}
        keys["PCT_EXC_OVERLAP"] = {"name": "Overlapping insert"}
        keys["PCT_EXC_CAPPED"] = {"name": "Over capped coverage"}

        # Config for the plot
        pconfig = {
            "id": "picard_wgs_metrics_bases",
            "title": "Picard: WGS Filtered Bases",
            "cpswitch": False,
            "ylab": "% Bases",
            "ymax": 100,
        }

        self.add_section(
            name="WGS Filtered Bases",
            anchor="picard-wgsmetrics-bases",
            description="For more information about the filtered categories, see the "
            + '<a href="http://broadinstitute.github.io/picard/picard-metric-definitions.html#CollectWgsMetrics.WgsMetrics" target="_blank">Picard documentation</a>.',
            plot=bargraph.plot(pdata, keys, pconfig),
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_wgsmetrics_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard MarkIlluminaAdapters """

import logging

from multiqc.plots import linegraph
from .util import read_histogram

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard MarkIlluminaAdapters reports and parse their data"""

    headers = ["clipped_bases", "read_count"]
    formats = [int, int]
    all_data = read_histogram(self, "picard/markilluminaadapters", "MarkIlluminaAdapters", headers, formats)

    # Filter to strip out ignored sample names
    all_data = self.ignore_samples(all_data)

    # Stop if we don't have anything
    if not all_data or len(all_data) == 0:
        return 0

    # Write parsed data to a file
    self.write_data_file(all_data, "multiqc_picard_mark_illumina_adapters")

    # Plot the data and add section
    pconfig = {
        "id": "picard_mark_illumina_adapters",
        "title": "Picard: Mark Illumina Adapters",
        "ylab": "Clipped Bases",
        "xlab": "Cycle Number",
        "xDecimals": False,
        "tt_label": "<b>Cycle {point.x}</b>: {point.y:.2f}",
        "ymin": 0,
    }

    lg = {}
    for s_name in all_data:
        lg[s_name] = {clipped_bases: data["read_count"] for clipped_bases, data in all_data[s_name].items()}

    self.add_section(
        name="Mark Illumina Adapters",
        description="""
            Number of Clipped Bases by Read.
            See the [Picard Docuementation](https://broadinstitute.github.io/picard/command-line-overview.html#MarkIlluminaAdapters) for details.
        """,
        anchor="picard-mark-illumina-adapters",
        plot=linegraph.plot(lg, pconfig),
    )

    # Return the number of detected samples to the parent module
    return len(all_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC geneBody_coverage.py
http://rseqc.sourceforge.net/#genebody-coverage-py """

from collections import OrderedDict
import logging

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find RSeQC gene_body_coverage reports and parse their data"""

    # Set up vars
    self.gene_body_cov_hist_counts = dict()
    self.gene_body_cov_hist_percent = dict()

    # TODO - Do separate parsing step to find skewness values
    # and add these to the general stats table?

    # Go through files and parse data
    for f in self.find_log_files("rseqc/gene_body_coverage"):

        # geneBodyCoverage.py
        # RSeQC >= v2.4
        # NB: Capitilisation
        if f["f"].startswith("Percentile"):
            keys = []
            nrows = 0
            for l in f["f"].splitlines():
                s = l.split("\t")
                # Check that this is the right file type (detection by capitilisation is pretty weak!)
                if len(s) < 3:
                    break
                if len(keys) == 0:
                    keys = s[1:]
                else:
                    nrows += 1
                    s_name = self.clean_s_name(s[0], f)
                    if s_name in self.gene_body_cov_hist_counts:
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                    self.add_data_source(f, s_name, section="gene_body_coverage")
                    self.gene_body_cov_hist_counts[s_name] = OrderedDict()
                    for k, var in enumerate(s[1:]):
                        self.gene_body_cov_hist_counts[s_name][int(keys[k])] = float(var)
            if nrows == 0:
                log.warning("Empty geneBodyCoverage file found: {}".format(f["fn"]))

        # geneBodyCoverage2.py
        #   AND
        # geneBodyCoverage.py
        # RSeQC < v2.4
        if f["f"].startswith("Total reads") or f["f"].startswith("percentile"):
            if f["s_name"] in self.gene_body_cov_hist_counts:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f["s_name"]))
            self.add_data_source(f, section="gene_body_coverage")
            self.gene_body_cov_hist_counts[f["s_name"]] = OrderedDict()
            nrows = 0
            for l in f["f"].splitlines():
                s = l.split("\t")
                # Check that this is the right file type (detection by capitilisation is pretty weak!)
                if len(s) > 2:
                    break
                try:
                    nrows += 1
                    self.gene_body_cov_hist_counts[f["s_name"]][int(s[0])] = float(s[1])
                except (IndexError, ValueError):
                    # Header lines don't use tabs, so get IndexError.
                    # Other unexpected weird stuff will not play nicely with int() and float()
                    pass
            if nrows == 0:
                del self.gene_body_cov_hist_counts[f["s_name"]]
                log.warning("Empty geneBodyCoverage file found: {}".format(f["fn"]))

    # Filter to strip out ignored sample names
    self.gene_body_cov_hist_counts = self.ignore_samples(self.gene_body_cov_hist_counts)

    if len(self.gene_body_cov_hist_counts) > 0:
        # Write data to file
        self.write_data_file(self.gene_body_cov_hist_counts, "rseqc_gene_body_cov")

        # Make a normalised percentage version of the data
        for s_name in self.gene_body_cov_hist_counts:
            self.gene_body_cov_hist_percent[s_name] = OrderedDict()
            total = sum(self.gene_body_cov_hist_counts[s_name].values())
            for k, v in self.gene_body_cov_hist_counts[s_name].items():
                self.gene_body_cov_hist_percent[s_name][k] = (v / total) * 100

        # Add line graph to section
        pconfig = {
            "id": "rseqc_gene_body_coverage_plot",
            "title": "RSeQC: Gene Body Coverage",
            "ylab": "% Coverage",
            "xlab": "Gene Body Percentile (5' -> 3')",
            "xmin": 0,
            "xmax": 100,
            "tt_label": "<strong>{point.x}% from 5'</strong>: {point.y:.2f}",
            "data_labels": [
                {"name": "Percentages", "ylab": "Percentage Coverage"},
                {"name": "Counts", "ylab": "Coverage"},
            ],
        }
        self.add_section(
            name="Gene Body Coverage",
            anchor="rseqc-gene_body_coverage",
            description='<a href="http://rseqc.sourceforge.net/#genebody-coverage-py" target="_blank">Gene Body Coverage</a>'
            " calculates read coverage over gene bodies."
            " This is used to check if reads coverage is uniform and"
            " if there is any 5' or 3' bias.",
            plot=linegraph.plot([self.gene_body_cov_hist_percent, self.gene_body_cov_hist_counts], pconfig),
        )

    # Return number of samples found
    return len(self.gene_body_cov_hist_counts)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC submodule to parse output from Samtools idxstats """

import logging
from collections import OrderedDict, defaultdict
from multiqc import config
from multiqc.plots import bargraph, linegraph

# Initialise the logger
log = logging.getLogger(__name__)


class IdxstatsReportMixin:
    def parse_samtools_idxstats(self):
        """Find Samtools idxstats logs and parse their data"""

        self.samtools_idxstats = dict()
        for f in self.find_log_files("samtools/idxstats"):
            parsed_data = parse_single_report(f["f"])
            if len(parsed_data) > 0:
                if f["s_name"] in self.samtools_idxstats:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f["s_name"]))
                self.add_data_source(f, section="idxstats")
                self.samtools_idxstats[f["s_name"]] = parsed_data

        # Filter to strip out ignored sample names
        self.samtools_idxstats = self.ignore_samples(self.samtools_idxstats)

        if len(self.samtools_idxstats) > 0:

            # Write parsed report data to a file (restructure first)
            self.write_data_file(self.samtools_idxstats, "multiqc_samtools_idxstats")

            # Prep the data for the plots
            keys = list()
            pdata = dict()
            pdata_norm = dict()
            pdata_obs_exp = dict()
            xy_counts = dict()
            # Count the total mapped reads for every chromosome
            chrs_mapped = defaultdict(lambda: 0)
            sample_mapped = defaultdict(lambda: 0)
            total_mapped = 0
            # Cutoff, can be customised in config
            cutoff = float(getattr(config, "samtools_idxstats_fraction_cutoff", 0.001))
            if cutoff != 0.001:
                log.info("Setting idxstats cutoff to: {}%".format(cutoff * 100.0))
            for s_name in self.samtools_idxstats:
                for chrom in self.samtools_idxstats[s_name]:
                    chrs_mapped[chrom] += self.samtools_idxstats[s_name][chrom][0]
                    sample_mapped[s_name] += self.samtools_idxstats[s_name][chrom][0]
                    total_mapped += self.samtools_idxstats[s_name][chrom][0]
            req_reads = float(total_mapped) * cutoff
            chr_always = getattr(config, "samtools_idxstats_always", [])
            if len(chr_always) > 0:
                log.info("Trying to include these chromosomes in idxstats: {}".format(", ".join(chr_always)))
            chr_ignore = getattr(config, "samtools_idxstats_ignore", [])
            if len(chr_ignore) > 0:
                log.info("Excluding these chromosomes from idxstats: {}".format(", ".join(chr_ignore)))
            xchr = getattr(config, "samtools_idxstats_xchr", False)
            if xchr:
                log.info('Using "{}" as X chromosome name'.format(xchr))
            ychr = getattr(config, "samtools_idxstats_ychr", False)
            if ychr:
                log.info('Using "{}" as Y chromosome name'.format(ychr))
            # Go through again and collect all of the keys that have enough counts
            # Also get the X/Y counts if we find them
            for s_name in self.samtools_idxstats:
                x_count = False
                y_count = False
                for chrom in self.samtools_idxstats[s_name]:
                    if float(chrs_mapped[chrom]) > req_reads or chrom in chr_always:
                        if chrom not in chr_ignore and chrom not in keys:
                            keys.append(chrom)
                    # Collect X and Y counts if we have them
                    mapped = self.samtools_idxstats[s_name][chrom][0]
                    if xchr is not False:
                        if str(xchr) == str(chrom):
                            x_count = mapped
                    else:
                        if chrom.lower() == "x" or chrom.lower() == "chrx":
                            x_count = mapped
                    if ychr is not False:
                        if str(ychr) == str(chrom):
                            y_count = mapped
                    else:
                        if chrom.lower() == "y" or chrom.lower() == "chry":
                            y_count = mapped
                # Only save these counts if we have both x and y
                if x_count and y_count:
                    xy_counts[s_name] = {"x": x_count, "y": y_count}
            # Ok, one last time. We have the chromosomes that we want to plot,
            # now collect the counts
            for s_name in self.samtools_idxstats:
                pdata[s_name] = OrderedDict()
                pdata_norm[s_name] = OrderedDict()
                pdata_obs_exp[s_name] = OrderedDict()
                genome_size = float(sum([stats[1] for stats in self.samtools_idxstats[s_name].values()]))
                for k in keys:

                    try:
                        pdata[s_name][k] = self.samtools_idxstats[s_name][k][0]
                        pdata_norm[s_name][k] = float(self.samtools_idxstats[s_name][k][0]) / sample_mapped[s_name]
                        chrom_size = float(self.samtools_idxstats[s_name][k][1])
                        expected_count = (chrom_size / genome_size) * float(sample_mapped[s_name])
                        pdata_obs_exp[s_name][k] = float(pdata[s_name][k]) / expected_count
                    except (KeyError, ZeroDivisionError):
                        pdata[s_name][k] = 0
                        pdata_norm[s_name][k] = 0
                        pdata_obs_exp[s_name][k] = 0

            # X/Y ratio plot
            if len(xy_counts) > 0:
                xy_keys = OrderedDict()
                xy_keys["x"] = {"name": xchr if xchr else "Chromosome X"}
                xy_keys["y"] = {"name": ychr if ychr else "Chromosome Y"}
                pconfig = {
                    "id": "samtools-idxstats-xy-plot",
                    "title": "Samtools idxstats: chrXY mapped reads",
                    "ylab": "Percent of X+Y Reads",
                    "cpswitch_counts_label": "Number of Reads",
                    "cpswitch_percent_label": "Percent of X+Y Reads",
                    "cpswitch_c_active": False,
                }
                self.add_section(
                    name="XY counts",
                    anchor="samtools-idxstats-xy-counts",
                    plot=bargraph.plot(xy_counts, xy_keys, pconfig),
                )

            # Mapped reads per chr line plot
            pconfig = {
                "id": "samtools-idxstats-mapped-reads-plot",
                "title": "Samtools idxstats: Mapped reads per contig",
                "ylab": "# mapped reads",
                "xlab": "Chromosome Name",
                "logswitch": True,
                "categories": True,
                "tt_label": "<strong>{point.category}:</strong> {point.y:.2f}",
                "data_labels": [
                    {"name": "Normalised Counts", "ylab": "Fraction of total count"},
                    {"name": "Observed over Expected Counts", "ylab": "log10 ( Observed over expected counts )"},
                    {"name": "Raw Counts", "ylab": "# mapped reads"},
                ],
            }
            self.add_section(
                name="Mapped reads per contig",
                anchor="samtools-idxstats",
                description="The <code>samtools idxstats</code> tool counts the number of mapped reads per chromosome / contig. "
                + "Chromosomes with &lt; {}% of the total aligned reads are omitted from this plot.".format(
                    cutoff * 100
                ),
                plot=linegraph.plot([pdata_norm, pdata_obs_exp, pdata], pconfig),
            )

        # Return the number of logs that were found
        return len(self.samtools_idxstats)


#  idxstats has four columns: chr, length, mapped, unmapped
#  http://www.htslib.org/doc/samtools.html


def parse_single_report(f):
    """Parse a samtools idxstats idxstats"""

    parsed_data = OrderedDict()
    for l in f.splitlines():
        s = l.split("\t")
        try:
            parsed_data[s[0]] = [int(s[2]), int(s[1])]
        except (IndexError, ValueError):
            pass
    return parsed_data



#!/usr/bin/env python
# coding=utf-8
from __future__ import print_function

import itertools
from multiqc.utils.util_functions import write_data_file
import re
from collections import defaultdict
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import table
from .utils import make_headers, Metric

# Initialise the logger
import logging

log = logging.getLogger(__name__)


NAMESPACE = "DRAGEN coverage"


class DragenCoverageMetrics(BaseMultiqcModule):
    def add_coverage_metrics(self):
        data_by_phenotype_by_sample = defaultdict(dict)

        for f in self.find_log_files("dragen/coverage_metrics"):
            s_name, data_by_phenotype = parse_coverage_metrics(f)
            s_name = self.clean_s_name(s_name, f)
            if s_name in data_by_phenotype_by_sample:
                log.debug(f"Duplicate sample name found! Overwriting: {s_name}")
            self.add_data_source(f, section="stats")
            data_by_phenotype_by_sample[s_name].update(data_by_phenotype)

        # Filter to strip out ignored sample names:
        data_by_phenotype_by_sample = self.ignore_samples(data_by_phenotype_by_sample)

        # Merge tumor and normal data:
        data_by_sample = defaultdict(dict)
        for sn in data_by_phenotype_by_sample:
            for phenotype in data_by_phenotype_by_sample[sn]:
                new_sn = sn
                if phenotype == "normal":
                    new_sn = sn + "_normal"
                data_by_sample[new_sn] = data_by_phenotype_by_sample[sn][phenotype]

        if not data_by_sample:
            return set()

        # Write data to file
        self.write_data_file(data_by_sample, "dragen_cov_metrics")

        all_metric_names = set()
        for sn, sdata in data_by_sample.items():
            for m in sdata.keys():
                all_metric_names.add(m)
        gen_stats_headers, own_tabl_headers = make_headers(all_metric_names, COV_METRICS)

        self.general_stats_addcols(data_by_sample, gen_stats_headers, namespace=NAMESPACE)

        self.add_section(
            name="Coverage metrics",
            anchor="dragen-cov-metrics",
            description="""
            Coverage metrics over a region (where the region can be a target region,
            a QC coverage region, or the whole genome). Press the `Help` button for details.
            """,
            helptext="""
            The following criteria are used when calculating coverage:

            * Duplicate reads and clipped bases are ignored.
            * Only reads with `MAPQ` > `min MAPQ` and bases with `BQ` > `min BQ` are considered

            Considering only bases usable for variant calling, _i.e._ excluding:

            1. Clipped bases
            2. Bases in duplicate reads
            3. Reads with `MAPQ` < `min MAPQ` (default `20`)
            4. Bases with `BQ` < `min BQ` (default `10`)
            5. Reads with `MAPQ` = `0` (multimappers)
            6. Overlapping mates are double-counted
            """,
            plot=table.plot(data_by_sample, own_tabl_headers, pconfig={"namespace": NAMESPACE}),
        )
        return data_by_sample.keys()


COV_METRICS = list(
    itertools.chain.from_iterable(
        [
            [
                Metric(
                    m.id,
                    m.title,
                    in_genstats=m.in_genstats,
                    in_own_tabl=m.in_own_tabl,
                    unit=m.unit,
                    descr=m.descr,
                    namespace=NAMESPACE,
                    precision=m.precision,
                ),
                Metric(
                    m.id.replace("{}", "genome"),
                    m.title.replace("{}", "genome"),
                    in_genstats=m.in_genstats,
                    in_own_tabl=m.in_own_tabl,
                    unit=m.unit,
                    descr=m.descr.replace("{}", "genome"),
                    namespace=NAMESPACE,
                    precision=m.precision,
                ),
                Metric(
                    m.id.replace("{}", "genome"),
                    m.title.replace("{}", "region"),
                    in_genstats=m.in_genstats,
                    in_own_tabl=m.in_own_tabl,
                    unit=m.unit,
                    descr=m.descr.replace("{}", "region"),
                    namespace=NAMESPACE,
                    precision=m.precision,
                ),
            ]
            for m in [
                # id_in_data                                            title (display name)  gen_stats  cov_table  unit  description  precision
                # Read stats:
                Metric("Aligned reads", "Aln reads", "hid", "#", "reads", "Total number of aligned reads."),
                Metric(
                    "Aligned reads in region",
                    "Reads on trg",
                    "hid",
                    "%",
                    "reads",
                    "Number of uniquely mapped reads to region relative to the number of uniquely mapped reads to the genome. When region is the target BED, this metric is equivalent to and replaces Capture Specificity based on target region.",
                ),
                Metric("Aligned bases", "Aln bases", "hid", "#", "bases", "Total number of aligned bases."),
                Metric(
                    "Aligned bases in region",
                    "Bases on trg",
                    "hid",
                    "%",
                    "bases",
                    "Number of uniquely mapped bases to the region relative to the number of uniquely mapped bases to the genome.",
                ),
                Metric(
                    "Average alignment coverage over {}",
                    "Depth",
                    "#",
                    "#",
                    "x",
                    "Coverage depth over {}: number of uniquely mapped bases to {} divided by the number of sites in {}.",
                ),
                Metric(
                    "Uniformity of coverage (PCT > 0.2*mean) over {}",
                    ">0.2×mean",
                    "hid",
                    "#",
                    "%",
                    "Percentage of sites with coverage greater than 20% of the mean coverage in region. Demonstrates the uniformity of coverage. The lower the better.",
                ),
                Metric(
                    "Average chr X coverage over {}",
                    "X cov",
                    None,
                    "hid",
                    "x",
                    "Average chromosome X coverage over {}. Calculated as the number of bases that aligned to the chromosome X (or to the intersection of chromosome X with the target region) divided by the total number of loci in the chromosome X (or the intersection with the target region). If there is no chromosome X in the reference genome or the region does not intersect chromosome X, this metric shows as NA.",
                ),
                Metric(
                    "Average chr Y coverage over {}",
                    "Y cov",
                    None,
                    "hid",
                    "x",
                    "Average chromosome Y coverage over {}. Calculated as the number of bases that aligned to the chromosome Y (or to the intersection of chromosome Y with the target region) divided by the total number of loci in the chromosome Y (or the intersection with the target region). If there is no chromosome Y in the reference genome or the region does not intersect chromosome Y, this metric shows as NA.",
                ),
                Metric(
                    "Average mitochondrial coverage over {}",
                    "MT cov",
                    None,
                    "hid",
                    "x",
                    "Average mitochondrial chromosome coverage over {}. Calculated as the number of bases that aligned to the mitochondrial chromosome (or to the intersection of mitochondrial chromosome with the target region) divided by the total number of loci in the mitochondrial chromosome (or the intersection with the target region). If there is no mitochondrial chromosome in the reference genome or the region does not intersect mitochondrial chromosome, this metric shows as NA.",
                ),
                Metric(
                    "Average autosomal coverage over {}",
                    "Mean aut cov",
                    None,
                    "hid",
                    "x",
                    "Average autosomal coverage over {}. Calculated as the number of bases that aligned to the autosomal loci in {} divided by the total number of loci in the autosomal loci in {}. If there is no autosomes in the reference genome, or the region does not intersect autosomes, this metric shows as NA.",
                ),
                Metric(
                    "Median autosomal coverage over {}",
                    "Med aut cov",
                    "hid",
                    "hid",
                    "x",
                    "Median alignment coverage over the autosomal loci in {}. If there is no autosome in the reference genome or the region does not intersect autosomes, this metric shows as NA.",
                ),
                Metric(
                    "Mean/Median autosomal coverage ratio over {}",
                    "Mean/med aut cov",
                    "hid",
                    "#",
                    "",
                    "Mean autosomal coverage in {} divided by the median autosomal coverage in {}. If there is no autosome in the reference genome or the region does not intersect autosomes, this metric shows as NA.",
                    precision=2,
                ),
                Metric(
                    "PCT of genome with coverage [  1x: inf)",
                    "⩾1x",
                    "hid",
                    "#",
                    "%",
                    "Percentage of sites in region with at least 1x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [  3x: inf)",
                    "⩾3x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 3x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 10x: inf)",
                    "⩾10x",
                    None,
                    "#",
                    "%",
                    "Percentage of sites in region with at least 10x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 15x: inf)",
                    "⩾15x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 15x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 20x: inf)",
                    "⩾20x",
                    "#",
                    "#",
                    "%",
                    "Percentage of sites in region with at least 20x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 50x: inf)",
                    "⩾50x",
                    "hid",
                    "#",
                    "%",
                    "Percentage of sites in region with at least 50x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [100x: inf)",
                    "⩾100x",
                    "hid",
                    "#",
                    "%",
                    "Percentage of sites in region with at least 100x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [  0x:  1x)",
                    "0x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with no coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [  1x:  3x)",
                    "1x..3x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 1x but less than 3x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [  3x: 10x)",
                    "3x..10x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 3x but less than 10x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 10x: 15x)",
                    "10x..15x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 10x but less than 15x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 15x: 20x)",
                    "15x..20x",
                    None,
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 15x but less than 20x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 20x: 50x)",
                    "20x..50x",
                    "hid",
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 20x but less than 50x coverage.",
                ),
                Metric(
                    "PCT of genome with coverage [ 50x:100x)",
                    "50x..100x",
                    "hid",
                    "hid",
                    "%",
                    "Percentage of sites in region with at least 50x but less than 100x coverage.",
                ),
            ]
        ]
    )
)


def parse_coverage_metrics(f):
    """
    T_SRR7890936_50pc.wgs_coverage_metrics_normal.csv
    T_SRR7890936_50pc.wgs_coverage_metrics_tumor.csv
    T_SRR7890936_50pc.target_bed_coverage_metrics.csv

    The coverage metrics report outputs a _coverage_metrics.csv file, which provides metrics over a region,
    where the region can be the genome, a target region, or a QC coverage region. The first column of the output
    file contains the section name COVERAGE SUMMARY and the second column is empty for all metrics.

    The following criteria are used when calculating coverage:
    - Duplicate reads and clipped bases are ignored.
    - Only reads with MAPQ > min MAPQ and bases with BQ > min BQ are considered

    COVERAGE SUMMARY,,Aligned bases,250311219292
    COVERAGE SUMMARY,,Aligned bases in genome,250311219292,100.00
    COVERAGE SUMMARY,,Average alignment coverage over genome,82.09
    COVERAGE SUMMARY,,Uniformity of coverage (PCT > 0.2*mean) over genome,95.58
    COVERAGE SUMMARY,,PCT of genome with coverage [100x: inf),22.67
    COVERAGE SUMMARY,,PCT of genome with coverage [ 50x: inf),89.33
    COVERAGE SUMMARY,,PCT of genome with coverage [ 20x: inf),95.44
    COVERAGE SUMMARY,,PCT of genome with coverage [ 15x: inf),95.68
    COVERAGE SUMMARY,,PCT of genome with coverage [ 10x: inf),95.92
    COVERAGE SUMMARY,,PCT of genome with coverage [  3x: inf),96.44
    COVERAGE SUMMARY,,PCT of genome with coverage [  1x: inf),96.91
    COVERAGE SUMMARY,,PCT of genome with coverage [  0x: inf),100.00
    COVERAGE SUMMARY,,PCT of genome with coverage [ 50x:100x),66.66
    COVERAGE SUMMARY,,PCT of genome with coverage [ 20x: 50x),6.10
    COVERAGE SUMMARY,,PCT of genome with coverage [ 15x: 20x),0.24
    COVERAGE SUMMARY,,PCT of genome with coverage [ 10x: 15x),0.24
    COVERAGE SUMMARY,,PCT of genome with coverage [  3x: 10x),0.52
    COVERAGE SUMMARY,,PCT of genome with coverage [  1x:  3x),0.47
    COVERAGE SUMMARY,,PCT of genome with coverage [  0x:  1x),3.09
    COVERAGE SUMMARY,,Average chr X coverage over genome,43.24
    COVERAGE SUMMARY,,Average chr Y coverage over genome,2.88
    COVERAGE SUMMARY,,Average mitochondrial coverage over genome,25675.54
    COVERAGE SUMMARY,,Average autosomal coverage over genome,85.00
    COVERAGE SUMMARY,,Median autosomal coverage over genome,83.79
    COVERAGE SUMMARY,,Mean/Median autosomal coverage ratio over genome,1.01
    COVERAGE SUMMARY,,Aligned reads,1689488809
    COVERAGE SUMMARY,,Aligned reads in genome,1689488809,100.00
    # where "genome" can be replaced with "region"

    Add metrics into gen stats table, plus add a dedicated table like VC
    """

    data = dict()

    for line in f["f"].splitlines():
        fields = line.split(",")
        metric = fields[2]
        value = fields[3]
        try:
            value = int(value)
        except ValueError:
            try:
                value = float(value)
            except ValueError:
                pass

        percentage = None
        if len(fields) > 4:  # percentage
            percentage = fields[4]
            try:
                percentage = float(percentage)
            except ValueError:
                pass

        data[metric] = value
        if percentage is not None:
            data[metric + " pct"] = percentage

    m = re.search(r"(.*)\.(\S*)_coverage_metrics_?(\S*)?.csv", f["fn"])
    sample, phenotype = m.group(1), m.group(2)
    return sample, {phenotype: data}
